/**
 * Define api of apeman-api-fs
 * @function api
 * @param {Object} [options] - Optional settings
 * @param {boolean} [options.logging] - Enable logger
 * @param {string} [options.root=process.cwd()] - Root directory path
 * @returns {Object}
 */
'use strict'

const co = require('co')
const { mkdirpAsync, existsAsync } = require('asfs')
const defaults = require('defaults')
const filemode = require('filemode')
const aglob = require('aglob')
const aslogger = require('aslogger')
const apemanApiPath = require('apeman-api-path')
const describe = require('./describe')

let modeString = (mode) => mode.toString(8)

/** @lends api */
function api (options = {}) {
  let logger = aslogger({ disabled: !options.logging })
  let { root } = defaults(options, {
    root: process.cwd()
  })

  let path = apemanApiPath({ root, logging: false })

  return {
    ping (pong = 'pong') {
      return co(function * () {
        logger.trace('pong:', pong)
        return pong
      })
    },
    glob (pattern) {
      const s = this
      return co(function * () {
        let filenames = yield aglob(pattern)
        for (let i = 0; i < filenames.length; i++) {
          let filename = filenames[ i ]
          filenames[ i ] = yield path.resolve.call(s, filename)
        }
        return filenames
      })
    },
    mkdir (dirname) {
      const s = this
      return co(function * () {
        let result = yield s.mkdirAll([ dirname ])
        return !!result[ 0 ]
      })
    },
    mkdirAll (dirnames) {
      const s = this
      return co(function * () {
        let results = []
        for (let dirname of dirnames) {
          let resolved = yield path.resolve.call(s, dirname)
          let exists = yield existsAsync(resolved)
          if (exists) {
            continue
          }
          yield mkdirpAsync(resolved)
          logger.debug(`Directory generated: ${dirname}`)
          results.push(dirname)
        }
        return results
      })
    },
    chmod (pattern, mode) {
      const s = this
      return s.chmodAll({ [pattern]: mode })
    },
    chmodAll (permissions) {
      const s = this
      return co(function * () {
        let results = []
        for (let pattern of Object.keys(permissions)) {
          let mode = permissions[ pattern ]
          let filenames = yield s.glob(pattern)
          for (let filename of filenames) {
            let changed = yield filemode(filename, mode, {})
            Object.keys(changed).forEach((filename) => {
              let { from, to } = changed[ filename ]
              logger.debug(`Permission changed: ${filename} ${modeString(from)} to ${modeString(to)}`)
              results.push(filename)
            })
          }
        }
        return results
      })
    },
    /**
     * A hook which runs before method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @returns {Promise}
     */
    $before (methodName, params) {
      const s = this
      let { state } = s
      return co(function * () {
        state.invokedAt = new Date()
        logger.info(`${methodName} started...`)
      })
    },
    /**
     * A hook which runs after method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @param {?*} returns - Return values
     * @returns {Promise}
     */
    $after (methodName, params, returns) {
      const s = this
      let { state } = s
      return co(function * () {
        let took = new Date() - state.invokedAt
        logger.info(`...${methodName} done! (${took}ms)`)
      })
    },
    /**
     * Method specifications
     */
    $spec: describe()
  }
}

module.exports = api
